
/*
 * $Revision: 1.5 $
 */

// Flash loader driver for the Philips LPC210x series microcontrollers.
 
#include <stdio.h>
#include <stdlib.h>
#include <iolpc210x.h>

#include "FlashPhilipsLPC210x.h"
#include "Interface.h" // The flash loader framework API declarations.


static IAP iap_entry = (IAP)kIAPentry; // MCU flash firmware interface function.
static unsigned long sectorbuf[SECTOR_SIZE/4]; // The sector buffer must be word aligned.
// The CPU clock speed (CCLK), the default value is used if no clock option is found.
static int clock = CCLK; 



// Execute a flash firmware command.
static int ExecuteCommand(unsigned long* cmd, unsigned long* status)
{
  int ret;
  
  for (;;)
  {
    iap_entry(cmd, status);
    ret = status[0];
    if (ret != STATUS_BUSY)
    {
      return ret;
    }
    // Try again if busy.
  }
}


// Program a sector. The device allows fractions of a sector to be written.
// dst  - the flash address to start writing at.
// src  - the address of the data buffer to be written.
// size - the number of bytes in the data buffer to write.
static void ProgramFlash(unsigned long dst, unsigned long* src, int size)
{
  int ret;
  int sector;
  int inhibit_erase = 0;
  unsigned long sum;
  unsigned long cmd[6];
  unsigned long status[3];

  // User Flash Mode. Interrupt vectors are not re-mapped and reside in Flash.
  MEMMAP = 1;

  sector = dst / SECTOR_SIZE;
  
#ifdef FLASH_256K
  // The 256k device have the following sector layout
  //     0-7   8k sectors
  //     8-9  64k sectors
  //    10-17  8k sectors
  // Map 8k sector number to device sector.
  if (sector >= 8 && sector <= 23)
  {
    // The 8k sector number is within the two 64k sectors.
    if (sector != 8 && sector != 16)
    {
      // Only erase a 64k sector for the first 8k part of a 64k sector.
      inhibit_erase = 1;
    }
    sector -= 8;
    sector /= 8;
    sector += 8;
  } else if (sector >= 24)
  {
    sector -= 14;
  }
#endif

  // Round up size to nearest allowable write size.
  if (size < 512)
    size = 512;
  if (size < 1024)
    size = 1024;
  if (size < 4096)
    size = 4096;
  if (size < 8192)
    size = 8192;

#ifdef FLASH_256K
  if (dst > 0x40000)
#else
  if (dst > 0x20000)
#endif
  {
    FlMessageBox("Data outside flash.");
    FlErrorExit();
  }

  // Check for sector 0 and that a reset vector are present,
  // note that the buffer is initialized to all 1's.
  if (sector == 0 && src[0] != 0xffffffff)
  {
    // Calculate exception vector 0x14 if the sector is zero and a reset address is present.
    sum = src[0] + src[1] + src[2] + src[3] +
          src[4] +          src[6] + src[7];
    src[5] = -sum; // Vector 0x14.
  }

  // Prepare sector for erase.
  cmd[0] = CMD_PREPARE_SECTORS;
  cmd[1] = sector;
  cmd[2] = sector + 1;
  ret = ExecuteCommand(cmd, status);
  if (ret != STATUS_CMD_SUCCESS)
  {
    FlMessageBox("CMD_PREPARE_SECTORS failed.");
    FlErrorExit();
  }

  if (!inhibit_erase)
  {
    // Erase sector.
    cmd[0] = CMD_ERASE_SECTORS;
    cmd[1] = sector;
    cmd[2] = sector + 1;
    cmd[3] = clock;
    ret = ExecuteCommand(cmd, status);
    if (ret != STATUS_CMD_SUCCESS)
    {
      FlMessageBox("CMD_ERASE_SECTORS failed.");
      FlErrorExit();
    }
  }

  // Prepare sector for write.
  cmd[0] = CMD_PREPARE_SECTORS;
  cmd[1] = sector;
  cmd[2] = sector + 1;
  ret = ExecuteCommand(cmd, status);
  if (ret != STATUS_CMD_SUCCESS)
  {
    FlMessageBox("CMD_PREPARE_SECTORS failed.");
    FlErrorExit();
  }

  // Program sector.
  cmd[0] = CMD_COPY_RAM_TO_FLASH;
  cmd[1] = dst;
  cmd[2] = (unsigned long)src;
  cmd[3] = size;
  cmd[4] = clock;
  ret = ExecuteCommand(cmd, status);
  if (ret != STATUS_CMD_SUCCESS)
  {
    FlMessageBox("CMD_COPY_RAM_TO_FLASH failed.");
    FlErrorExit();
  }
}


// Write one byte to flash at addr.
// The bytes are buffered in sectorbuf. The sectorbuf is written to
// the flash when it overflows.
// If byte == -1 the flash loader framework signals a flush operation
// at the end of the input file.
static void FlashWriteByte(unsigned long addr, int byte)
{
  int i;
  int offset;           // Current offset into sector buffer.
  static int bytes = 0; // Number of bytes in the sector buffer (including gaps).
  static int base_addr; // Pysical address of the start of the sector.

restart:
  if (bytes == 0) // First byte of a new sector.
  {
    bytes = addr % SECTOR_SIZE;
    base_addr = addr - bytes; // Calculate physical start address of this sector.
    // Intialize sector buffer with 0xff (flash erase pattern).
    for (i = 0; i < SECTOR_SIZE/4; i++)
      sectorbuf[i] = 0xffffffff;
  }

  // Write sector buffer to the flash memory if a flush is requested or the
  // new byte address is beyond the sector buffer.
  if (byte == -1 || addr - base_addr >= SECTOR_SIZE)
  {
    ProgramFlash(base_addr, sectorbuf, bytes);
    if (byte == -1)
    {
      return; // This was a flush operation, all data is written.
    }
    
    bytes = 0;
    // Previous sector buffer is written to flash.
    // Continue with a new empty sector buffer.
    goto restart;
  }

  // Store byte in sector buffer.
  offset = addr - base_addr;
  ((unsigned char*)sectorbuf)[offset] = byte;
  bytes = offset + 1;
}


void FlashDriverInitialize(int argc, char const* argv[])
{
  const char* str;
  
  // Register the flash write function.
  FlRegisterWriteFunction(FlashWriteByte);

  // See if user has passed a clock speed option.
  // If not, the default CCLK value is used.
  str = FlFindOption("--clock", 1, argc, argv);
  if (str)
  {
    clock = strtoul(str, 0, 0);
  }
}

