
/*
 * $Revision: 1.1 $
 */
 
#include <stdio.h>
#include <stdlib.h>

#include "SimpleCode.h"
#include "Interface.h"
#include "Private.h"

unsigned long baseAddress;        // The flash base address to start flashing at
int overrideBaseAddress = 0;      // if default given by input file is overridden.

static int file_fd;
static unsigned long checksum = 0;


// Reads an unsigned integer from file. The size parameter
// is the number of bytes of the integer.
static unsigned long ReadUint(int size)
{
  int i;
  int c;
  unsigned long value = 0;

  // Loop over all bytes to be read and shift them in.
  for (i = 0; i < size; i++)
  {
    c = FlFileReadByte(file_fd);
    if (c == -1)
    {
      FlMessageBox("Found unexpected EOF in input file.");
      FlErrorExit();
    }
    
    value <<= 8;
    value |= c;
    checksum += c; // Calculate checksum.
  }
  return value;
}


void SimpleCodeProcessFile(int fd, void (*wr_byte_func)(unsigned long addr, int byte))
{
  SimpleCodeHeader hdr;
  SimpleCodeDataRec drec;
  int record_tag;
  int proceed;
  int i;
  int tick;
  int old_tick = 0;
  int progress_bar = 0;
  int bytes_written = 0;
  int record_number = 0;
  int record_error = 0;
  unsigned long start_offset = 0;
  unsigned long curr_addr;
  unsigned long saved_checksum;
  unsigned char byte;

  file_fd = fd; // Save file descriptor (used by ReadUint function).

  // Read file header.
  hdr.magic = ReadUint(4);
  hdr.flags = ReadUint(4);
  hdr.bytes = ReadUint(4);
  hdr.version = ReadUint(2);

  if (hdr.magic != 0x7f494152)
  {  
    FlMessageBox("The input file is not in the simple-code format.");
    FlErrorExit();
  }

  // Loop over all records.
  for (proceed = 1; proceed; )
  {
    // Read record tag.
    record_tag = ReadUint(1);
    switch (record_tag)
    {
    case 1: // Data record.
      if (!overrideProgressBar && !progress_bar)
      {
        FlProgressBarCreate("Programming flash");
        progress_bar = 1;
      }
      
      drec.segtype = ReadUint(1);
      drec.flags = ReadUint(2);
      drec.start = ReadUint(4);
      drec.bytes = ReadUint(4);

      // Check if the user wants to override the load address.
      if (overrideBaseAddress && record_number == 0)
      {
        start_offset = baseAddress - drec.start;
      }
      curr_addr = drec.start + start_offset;

      // Loop over all data bytes in the record.
      for (i = 0; i < drec.bytes; i++)
      {
        byte = ReadUint(1);
        (*wr_byte_func)(curr_addr++, byte); // Send byte to flash loader driver.
        bytes_written++;

        if (!overrideProgressBar)
        {
          // Limit the number of progress bar updates to 10 to keep the
          // number of JTAG transactions as low as possible to not waste time.
          tick = (bytes_written * 10) / hdr.bytes;
          if (tick != old_tick)
          {
            FlProgressBarUpdate(tick * 10); // Progress value is 0..100.
            old_tick = tick;
          }
        }
      }
      record_number++;
      break;
    case 2: // Entry record (not used in flash loader).
      ReadUint(4);
      ReadUint(1);
      break;
    case 3: // End record.
      // The checksum itself is not part of the calculated checsum.
      saved_checksum = checksum; // Save calculated checksum before reading checksum from file.
      checksum = ReadUint(4);
      checksum += saved_checksum; // Should be zero, verified below.
      proceed = 0;
      break;
    default:
      record_error = 1;
      break;
    }
  }
  (*wr_byte_func)(0, -1); // Flush remaining buffered data in flash loader driver.

  // Destroy the progress bar.
  if (!overrideProgressBar && progress_bar)
  {
    FlProgressBarDestroy();
  }

  if (record_error)
  {
    // The record error is displayed after closing the progress bar to avoid
    // user confusion.
    FlMessageBox("Unknown record in simple-code input file.");
    FlErrorExit();
  }

  if (checksum)
  {
    // The checsum error is displayed after closing the progress bar to avoid
    // user confusion.
    FlMessageBox("Checksum error in simple-code input file.");
    FlErrorExit();
  }
}


